use std::any::Any;
use std::collections::HashMap;

use crate::drivers::DatabaseDriver;

#[derive(Debug)]
enum QueryClauseType {
    AndGroupStart,
    GroupEnd,
    GroupStart,
    Like,
    OrGroupStart,
    Where,
    WhereIn,
}

#[derive(Debug)]
struct QueryClause {
    clause_type: QueryClauseType,
    conjunction: String,
    string: String,
}

#[derive(Default, Debug)]
struct QueryState {
    select_string: String,
    from_string: String,
    set_string: String,
    order_string: String,
    group_string: String,

    // Keys for insert/update statement
    pub set_array_keys: Vec<String>,

    // Order by clause
    order_array: HashMap<String, String>,

    // Group by clause
    group_array: HashMap<String, String>,

    // Values to apply to prepared statements
    pub values: Vec<String>,

    // Values to apply to where clauses in prepared statements
    pub where_values: Vec<String>,

    limit: u32,

    offset: u32,

    // Query components for complex selects
    query_map: Vec<QueryClause>,

    // Query components for having clauses
    having_map: Vec<QueryClause>,
}

impl QueryState {
    pub fn new() -> QueryState {
        QueryState::default()
    }
}

#[derive(Default, Debug)]
pub struct QueryBuilder {
    state: QueryState,
    driver: Option<DatabaseDriver>,
}

impl QueryBuilder {
    pub fn new() -> QueryBuilder {
        QueryBuilder {
            state: QueryState::new(),
            driver: None,
        }
    }

    pub fn set(mut self, key: String, value: String) -> Self {
        self.state.set_array_keys.push(key);
        self.state.values.push(value);

        self
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn set_key_value() {
        let builder = QueryBuilder::new();
        let builder = builder.set("foo".to_string(), "bar".to_string());

        assert_eq!(builder.state.values, vec!["bar"]);
    }
}